//PassMark PluginTest (Part of BurnInTest)
//Copyright 2005
//
//Program:  PluginTest
//Author:   Ian Robinson
//Module:   PluginTestInterface.cpp
//
//Description:  
//	Shared memory interface definition between BurnInTest and the external plugin Test
//
//History
//  12-Dec-05 First version used for ModemTest integration to BurnInTest as a Plugin
//		usage (when STANDALONE defined)eg
//		BIT_PLUGIN_INT0 C:\Passmark\Software\ModemTest\Debug\ModemTest.exe /p 1 results.log C:\Passmark\Software\ModemTest\Debug\ModemTest.cfg
//  18-2-05 Updated for version 2 of PLUGININTERFACE
//  03-9-04 Module created
//
#include <Windows.h>
#include <stdio.h>
#include "PluginTestInterface.h"

#ifdef STANDALONE
static PLUGININTERFACE sBitInterface; 
#endif


void strn_clean_cpy (char * szTextOut, char * szTextIn, int iStrLen);

//-----------------------------------------------------------------------------
// Name: CPlugIn()
// Desc: Plugin class constructor
//-----------------------------------------------------------------------------
CPlugIn::CPlugIn()
{
}


//-----------------------------------------------------------------------------
// Name: ~CPlugIn()
// Desc: Plugin class destructor
//-----------------------------------------------------------------------------
CPlugIn::~CPlugIn()
{
}

bool CPlugIn::PluginRunning()
{
	return (m_BitInterface->IN_TestRunning == TRUE);
}


bool CPlugIn::InitPluginInterface(char * argv0)
{

#ifdef STANDALONE
	//For stand alone debugging
	m_BitInterface = &sBitInterface;
	m_BitInterface->IN_TestRunning = true;
	m_BitInterface->IN_DutyCycle = 100;
#else

	//Get name of shared memory
	strcpy_s(m_SharedMemInterface, MAXCOMMANDLINE, argv0);
	if (m_SharedMemInterface[0] != 'B' && m_SharedMemInterface[1] != 'I')
	{
		return false;
	}

	//Open shared memory interface
	m_hMappedObject = OpenFileMapping (FILE_MAP_ALL_ACCESS, FALSE, m_SharedMemInterface);
	if (m_hMappedObject == NULL)
	{
		//DWORD dwRet = GetLastError();
		return false;
	}

	m_BitInterface = (PLUGININTERFACE *) MapViewOfFile(m_hMappedObject,FILE_MAP_ALL_ACCESS,0,0,0);
	if (m_BitInterface == NULL) 
	{
		CloseHandle (m_hMappedObject);
		return false;
	}
#endif

	//Set initial test values
	m_BitInterface->OUT_Plugin_interface_version = PLUGIN_INTERFACE_VERSION;
	m_BitInterface->OUT_iCycle = 0;
	m_BitInterface->OUT_iErrorCount = 0;					//Total Errors
	m_BitInterface->OUT_iErrorSeverity = 0;
	m_BitInterface->OUT_i64WriteOps = 0;					//Total operations

	m_BitInterface->OUT_iStatus = PLUGIN_NOTUSED;
	m_BitInterface->OUT_i64ReadOps = PLUGIN_NOTUSED;
	m_BitInterface->OUT_i64VerifyOps = PLUGIN_NOTUSED;
	m_BitInterface->OUT_bUserDef1 = false;
	m_BitInterface->OUT_bNewUserDefVal1 = false;
	m_BitInterface->OUT_bUserDef2 = false;
	m_BitInterface->OUT_bNewUserDefVal2 = false;
	m_BitInterface->OUT_bNewError = false;		
	m_BitInterface->OUT_bNewStatus = false;		
	m_BitInterface->OUT_bTestStopped = false;	

	// Set the labels that will be displayed on the BurnInTest Plugin Test window
	strn_clean_cpy(m_BitInterface->OUT_szWindowTitle,"ModemTest", PLUGIN_MAXDISPLAYTEXT);	
	strn_clean_cpy(m_BitInterface->OUT_szStatus,"Starting", PLUGIN_MAXDISPLAYTEXT);				
	strn_clean_cpy(m_BitInterface->OUT_szError,"", PLUGIN_MAXERRORTEXT);						
	strn_clean_cpy(m_BitInterface->OUT_szWriteOps, "Operations:", PLUGIN_MAXDISPLAYTEXT);	
	strn_clean_cpy(m_BitInterface->OUT_szReadOps,"", PLUGIN_MAXDISPLAYTEXT);							
	strn_clean_cpy(m_BitInterface->OUT_szVerifyOps, "", PLUGIN_MAXDISPLAYTEXT);
	m_BitInterface->OUT_bDisplayTextSet = true;

	return true;
}

void CPlugIn::Invalid_plugin_cmd_line()
{
	char	szTempErr[PLUGIN_MAXERRORTEXT];

	m_BitInterface->OUT_iErrorCount++;
	m_BitInterface->OUT_iErrorSeverity = ERRORWARNING;
	sprintf_s(szTempErr, sizeof(szTempErr), "Invalid ModemTest.exe command line. Usage: ModemTest /b <COM port number> <log file> <config. file>");

	strn_clean_cpy(m_BitInterface->OUT_szError, szTempErr, PLUGIN_MAXERRORTEXT);
	m_BitInterface->OUT_bNewError = true;
	wait_plugin (2000);	//Give BurnINTest a chance to read the error before closing
}


void CPlugIn::UpdateBITStatus(char *line)
{
	strn_clean_cpy(m_BitInterface->OUT_szStatus, line, PLUGIN_MAXDISPLAYTEXT);
	m_BitInterface->OUT_bNewStatus = true;		//flag that a new status string is available
}


void CPlugIn::IncrementNumOperations(int iNumOps)
{
	m_BitInterface->OUT_i64WriteOps += iNumOps;	//Update the number of operations
}


void CPlugIn::IncrementNumErrors(char * szTmpError, int iBITErrorSeverity)
{
	char	szTempErr[PLUGIN_MAXERRORTEXT];

	m_BitInterface->OUT_iErrorCount++;
	m_BitInterface->OUT_iErrorSeverity = iBITErrorSeverity;
	sprintf_s(szTempErr, sizeof(szTempErr), szTmpError);
	strn_clean_cpy(m_BitInterface->OUT_szError, szTempErr, PLUGIN_MAXERRORTEXT);
	m_BitInterface->OUT_bNewError = true;
	wait_plugin (2000);	//Give BurnINTest a chance to read the error before closing
	return;
}

void CPlugIn::IncrementCycle()
{
	m_BitInterface->OUT_iCycle += 1;	//Update the BIT cycle counter
}


//Name:	wait_plugin
//Waits for a set period of time or until the thread must end
//Notes
// This avoids a long wait when the user stops all the tests.
void CPlugIn::wait_plugin(int wait_time) 
{

	while (wait_time > 0) 
	{
		Sleep (100);
		wait_time -= 100;
		if (!m_BitInterface->IN_TestRunning)
			break;
	}

}


//strn_clean_cpy 
// 
// Use this function to cpy any string to the BurnInTest Interface to ensure bonudary lengths are not exceeded
//
// does a strncpy, but cleans the string up first (removes funny characters, '%' and '\' and make them spaces)
//
void CPlugIn::strn_clean_cpy (char * szTextOut, char * szTextIn, int iStrLen)
{
	char	szTextTemp[PLUGIN_MAXERRORTEXT+1];
	bool	bEndString = false;

	if (iStrLen >= PLUGIN_MAXERRORTEXT)
		iStrLen  = PLUGIN_MAXERRORTEXT - 1;

	for (int i = 0; i < iStrLen; i++)
	{
		if (szTextIn[i] == '\0' || bEndString)
		{
			szTextTemp[i] = '\0';
			bEndString = true;
		}
		else if (szTextIn[i] < 0x020)
			szTextTemp[i] = ' ';
		else if (szTextIn[i] == '%')
			szTextTemp[i] = ' ';
		else if (szTextIn[i] == '\\')
			szTextTemp[i] = ' ';
		else
			szTextTemp[i] = szTextIn[i];
	}
	szTextTemp[iStrLen-1] = '\0'; //Make sure its terminated

	strcpy_s (szTextOut, iStrLen, szTextTemp);
	szTextOut[iStrLen-1] = '\0'; 
}
